<?php

namespace Drupal\commerce_vtbpayment\Plugin\Commerce\PaymentGateway;

require dirname(__DIR__, 4) . '/vendor/autoload.php';

use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\commerce_payment\Exception\PaymentGatewayException;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OffsitePaymentGatewayBase;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsNotificationsInterface;
use Drupal\commerce_payment\PaymentMethodTypeManager;
use Drupal\commerce_payment\PaymentTypeManager;
use Drupal\Component\Datetime\TimeInterface;
use Drupal\commerce_vtbpayment\Classes\Api\VtbApi;
use Drupal\commerce_vtbpayment\Classes\Common\VtbPayLogger;
use Drupal\commerce_vtbpayment\Classes\Exception\VtbPayException;
use Symfony\Component\HttpFoundation\Request;

/**
 * Provides the Off-site Vtb payment gateway.
 *
 * @CommercePaymentGateway(
 *   id = "commerce_vtbpayment",
 *   label = "VtbPay",
 *   display_label = "VtbPay",
 *   forms = {
 *     "offsite-payment" = "Drupal\commerce_vtbpayment\PluginForm\OffsiteRedirect\VtbPaymentOffsiteForm",
 *   },
 *   payment_method_types = {"credit_card"},
 *   credit_card_types = {
 *     "MIR", "mastercard", "visa",
 *   },
 *   requires_billing_information = FALSE,
 * )
 */
class Vtbpayment extends OffsitePaymentGatewayBase implements SupportsNotificationsInterface
{
    private string $client_id;
    private string $client_secret;
    private string $merchant_authorization;
    private bool $mode;
    private bool $logging;

    // Fiscal settings
    private bool $fiscal_enable;
    private string $fiscal_email;
    private string $fiscal_payment_type_delivery;
    private string $fiscal_measure;
    private string $fiscal_tax_type;
    private string $fiscal_payment_type;
    private string $fiscal_subject;

    // Common
    private object $logger;



    public function __construct(
        array $configuration,
        $plugin_id,
        $plugin_definition,
        EntityTypeManagerInterface $entity_type_manager,
        PaymentTypeManager $payment_type_manager,
        PaymentMethodTypeManager $payment_method_type_manager,
        TimeInterface $time
    ) {
        parent::__construct(
            $configuration,
            $plugin_id,
            $plugin_definition,
            $entity_type_manager,
            $payment_type_manager,
            $payment_method_type_manager,
            $time
        );

        $this->setConfigurationProps($configuration);
    }


    /**
     * Устанавливает свойства настроек
     * @param array $configuration - массив настроек
     */
    private function setConfigurationProps(array $configuration)
    {
        // Bool props
        $bool_settings = [
            'logging',
            'mode',
            'fiscal_enable'
        ];
        foreach ($bool_settings as $setting) {
            $bool_setting = $configuration[$setting] ?? false;
            $this->{$setting} = (bool) ($bool_setting === 'test' || $bool_setting === '1');
        }

        // Text props
        $text_settings = [
            'client_id',
            'client_secret',
            'merchant_authorization',
            'fiscal_email',
            'fiscal_payment_type_delivery',
            'fiscal_measure',
            'fiscal_tax_type',
            'fiscal_payment_type',
            'fiscal_subject'
        ];

        foreach ($text_settings as $setting) {
            $this->{$setting} = $configuration[$setting] ?? '';
        }
    }


    /**
     * Дефолтные значения формы настроек модуля.
     * @return array - возвращает дефолные значения
     */
    public function defaultConfiguration()
    {
        return [
                'client_id' => '',
                'client_secret' => '',
                'merchant_authorization' => '',
                'logging' => true,
                'fiscal_enable' => false,
                'fiscal_email' => '',
                'fiscal_payment_type_delivery' => 'full_prepayment',
                'fiscal_measure' => 0,
                'fiscal_tax_type' => 'none',
                'fiscal_payment_type' => 'full_prepayment',
                'fiscal_subject' => 1
            ] + parent::defaultConfiguration();
    }


    /**
     * Создание формы настроек модуля.
     * @param array $form - массив параметров настроек
     * @param object $form_state - объект формы настроек
     * @return void
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state)
    {
        $form = parent::buildConfigurationForm($form, $form_state);

        $form['client_id'] = [
            '#title' => $this->t('Client ID'),
            '#description' => $this->t('Для вызова API требуется идентификатор клиента продавца.'),
            '#default_value' => $this->configuration['client_id'],
            '#type' => 'textfield',
            '#required' => true,
        ];

        $form['client_secret'] = [
            '#title' => $this->t('Client Secret'),
            '#description' => $this->t('Для вызова API требуется секретный ключ продавца.'),
            '#default_value' => $this->configuration['client_secret'],
            '#type' => 'textfield',
            '#required' => true,
        ];

        $form['merchant_authorization'] = [
            '#title' => $this->t('Merchant-Authorization'),
            '#description' => $this->t('Заголовок запроса, используемый ресурсом мерчанта для дополнительной аутентификации при доступе к платежному шлюзу.'),
            '#default_value' => $this->configuration['merchant_authorization'],
            '#type' => 'textfield',
        ];

        $form['logging'] = [
            '#title' => $this->t('Логирование'),
            '#description' => $this->t(''),
            '#default_value' => $this->configuration['logging'] ?: false,
            '#type' => 'radios',
            '#options' =>  [
                true => 'Включено',
                false => 'Выключено'
            ]
        ];

        $form['fiscal_enable'] = [
            '#title' => $this->t('Включить фискализацию'),
            '#description' => $this->t('Включение и отключение фискализации.'),
            '#default_value' => $this->configuration['fiscal_enable'] ?: false,
            '#type' => 'radios',
            '#options' =>  [
                true => 'Включено',
                false => 'Выключено'
            ]
        ];

        $form['fiscal_email'] = [
            '#title' => $this->t('Электронная почта'),
            '#description' => $this->t('Электронная почта для отправки фискального чека, если не указана почта покупателя.'),
            '#default_value' => $this->configuration['fiscal_email'],
            '#type' => 'textfield',
        ];

        $form['fiscal_payment_type_delivery'] = [
            '#title' => $this->t('Способ расчета для доставки'),
            '#description' => $this->t('Способ расчета для услуги доставки.'),
            '#default_value' => $this->configuration['fiscal_payment_type_delivery'] ?: 'full_prepayment',
            '#type' => 'select',
            '#options' => $this->getFiscalPaymentOptions(),
        ];

        $form['fiscal_measure'] = [
            '#title' => $this->t('Единицы измерения'),
            '#description' => $this->t('Единицы измерения по умолчанию (для товаров и услуг, у которых не указана единица измерения).'),
            '#default_value' => $this->configuration['fiscal_measure'] ?: 0,
            '#type' => 'select',
            '#options' => $this->getFiscalMeasureOptions(),
        ];

        $form['fiscal_tax_type'] = [
            '#title' => $this->t('Налоговая ставка'),
            '#description' => $this->t('Ставка налогообложения по умолчанию (для товаров и услуг, у которых не указана ставка налогообложения).'),
            '#default_value' => $this->configuration['fiscal_tax_type'] ?: 'none',
            '#type' => 'select',
            '#options' => $this->getFiscalTaxOptions(),
        ];

        $form['fiscal_payment_type'] = [
            '#title' => $this->t('Способ расчета'),
            '#description' => $this->t('Способ расчета по умолчанию (для товаров и услуг, у которых не указан способ расчета).'),
            '#default_value' => $this->configuration['fiscal_payment_type'] ?: 'full_prepayment',
            '#type' => 'select',
            '#options' => $this->getFiscalPaymentOptions(),
        ];

        $form['fiscal_subject'] = [
            '#title' => $this->t('Предмет расчета'),
            '#description' => $this->t('Предмет расчета по умолчанию (для товаров и услуг, у которых не указан предмет расчета).'),
            '#default_value' => $this->configuration['fiscal_subject'] ?: 1,
            '#type' => 'select',
            '#options' => $this->getFiscalSubjectOptions(),
        ];

        return $form;
    }


    /**
     * Сохранение параметров настроек модуля.
     * @param array $form - массив параметров настроек
     * @param object $form_state - объект формы настроек
     * @return void
     */
    public function submitConfigurationForm(array &$form, FormStateInterface $form_state)
    {
        parent::submitConfigurationForm($form, $form_state);

        if (!$form_state->getErrors()) {
            $settings = [
                'client_id',
                'client_secret',
                'merchant_authorization',
                'logging',
                'fiscal_enable',
                'fiscal_email',
                'fiscal_payment_type_delivery',
                'fiscal_measure',
                'fiscal_tax_type',
                'fiscal_payment_type',
                'fiscal_subject'
            ];
            $values = $form_state->getValue($form['#parents']);
            foreach ($settings as $setting) {
                $this->configuration[$setting] = $values[$setting] ?? '';
            }
        }
    }


    function getFiscalPaymentOptions()
    {
        return [
            'full_prepayment' => $this->t('Предоплата 100%'),
            'prepayment' => $this->t('Предоплата'),
            'advance' => $this->t('Аванс'),
            'full_payment' => $this->t('Полный расчет'),
            'partial_payment' => $this->t('Частичный расчет и кредит'),
            'credit' => $this->t('Передача в кредит'),
            'credit_payment' => $this->t('Оплата кредита')
        ];
    }


    function getFiscalMeasureOptions()
    {
        return [
            0 => $this->t('Применяется для предметов расчета, которые могут быть реализованы поштучно или единицами'),
            10 => $this->t('Грамм'),
            11 => $this->t('Килограмм'),
            12 => $this->t('Тонна'),
            20 => $this->t('Сантиметр'),
            21 => $this->t('Дециметр'),
            22 => $this->t('Метр'),
            30 => $this->t('Квадратный сантиметр'),
            31 => $this->t('Квадратный дециметр'),
            32 => $this->t('Квадратный метр'),
            40 => $this->t('Миллилитр'),
            41 => $this->t('Литр'),
            42 => $this->t('Кубический метр'),
            50 => $this->t('Киловатт час'),
            51 => $this->t('Гигакалория'),
            70 => $this->t('Сутки (день)'),
            71 => $this->t('Час'),
            72 => $this->t('Минута'),
            73 => $this->t('Секунда'),
            80 => $this->t('Килобайт'),
            81 => $this->t('Мегабайт'),
            82 => $this->t('Гигабайт'),
            83 => $this->t('Терабайт'),
            255 => $this->t('Применяется при использовании иных единиц измерения')
        ];
    }


    function getFiscalTaxOptions()
    {
        return [
            'none' => $this->t('Без НДС'),
            'vat0' => $this->t('НДС по ставке 0%'),
            'vat10' => $this->t('НДС чека по ставке 10%'),
            'vat110' => $this->t('НДС чека по расчетной ставке 10/110'),
            'vat20' => $this->t('НДС чека по ставке 20%'),
            'vat120' => $this->t('НДС чека по расчетной ставке 20/120')
        ];
    }


    function getFiscalSubjectOptions()
    {
        return [
            1 => $this->t('Товар'),
            2 => $this->t('Подакцизный товар'),
            3 => $this->t('Работа'),
            4 => $this->t('Услуга'),
            5 => $this->t('Ставка азартной игры'),
            6 => $this->t('Выигрыш азартной игры'),
            7 => $this->t('Ставка лотереи'),
            8 => $this->t('Выигрыш лотереи'),
            9 => $this->t('Предоставление прав'),
            10 => $this->t('Платеж'),
            11 => $this->t('Агентское вознаграждение'),
            12 => $this->t('Выплата'),
            13 => $this->t('Иной предмет расчёта'),
            14 => $this->t('Имущественное право'),
            15 => $this->t('Внереализационный доход'),
            16 => $this->t('Страховые взносы'),
            17 => $this->t('Торговый сбор'),
            18 => $this->t('Курортный сбор'),
            19 => $this->t('Залог'),
            20 => $this->t('Расход'),
            21 => $this->t('Взносы на ОПС ИП'),
            22 => $this->t('Взносы ОПС'),
            23 => $this->t('Взносы на ОМС ИП'),
            24 => $this->t('Взносы на ОМС'),
            25 => $this->t('Взносы на ОСС'),
            26 => $this->t('Платеж казино'),
            27 => $this->t('Выдача денежных средств банковским платежным агентом'),
            30 => $this->t('Подакцизный товар, подлежащий маркировке средством идентификации, не имеющий кода маркировки'),
            31 => $this->t('Подакцизный товар, подлежащий маркировке средством идентификации, имеющий код маркировки'),
            32 => $this->t('Товар, подлежащий маркировке средством идентификации, не имеющий кода маркировки, за исключением подакцизного товара'),
            33 => $this->t('Товар, подлежащий маркировке средством идентификации, имеющий код маркировки, за исключением подакцизного товара')
        ];
    }


    /**
     * Обработчик cancel, вызываемый при переходе на страницу cancel_url, после попытки оплаты.
     * Показывает сообщение что заказ был отклонен
     * @param object $order - объект заказа
     * @param object $request - объект запроса к странице возврата
     * @return void
     */
    public function onCancel(OrderInterface $order, Request $request)
    {
        \Drupal::messenger()->addMessage($this->t(
            'Вы отменили оплату заказа с помощью @gateway,
             но можете возобновить процесс оформления заказа здесь, когда будете готовы.', [
            '@gateway' => $this->getDisplayLabel(),
        ]));
        throw new PaymentGatewayException('Payment cancelled by user');
    }


    /**
     * Обработчик return, вызываемый при переходе на страницу return_url, после попытки оплаты.
     * Добавляет информацию о платеже в заказ
     * @param object $order - объект заказа
     * @param object $request - объект запроса к странице возврата
     * @return void
     */
    public function onReturn(OrderInterface $order, Request $request)
    {
        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'return-' . rand(1111, 9999));
        // Logging $_REQUEST
        $this->logger->debug(
            __FUNCTION__ . ' > return: ', [
            'request_data' => $_REQUEST
        ]);

        try {
            $remote_state = $this->getPaymentStatus($order->id());

            $message = '';
            if ($remote_state == 'PAID') {
                $payment_data = [
                    'state' => 'completed',
                    'amount' => $order->getBalance(),
                    'payment_gateway' => $this->parentEntity->id(),
                    'order_id' => $order->id(),
                    'remote_id' => $request->query->get('orderId'),
                    'remote_state' => $remote_state,
                ];
                $message = 'Ваш платеж прошел успешно для заказа @orderid';
            }
            elseif ($remote_state == 'PENDING') {
                $payment_data = [
                    'state' => 'authorization',
                    'amount' => $order->getBalance(),
                    'payment_gateway' => $this->parentEntity->id(),
                    'order_id' => $order->id(),
                    'remote_id' => $request->query->get('orderId'),
                    'remote_state' => $remote_state,
                ];
                $message = 'Ваш платеж для заказа @orderid ожидает обработки';
            }
            else {
                throw new VtbPayException('Платеж не был выполнен.', [
                    'remote_state' => $remote_state
                ]);
            }

            $this->entityTypeManager->getStorage('commerce_payment')
            ->create($payment_data)->save();

            \Drupal::messenger()->addMessage($this->t(
                $message, [
                '@orderid' => $order->id()
            ]));
        } catch (\Exception | VtbPayException $e) {
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $msg = $e->getMessage() ?: 'Ошибка проверки статуса оплаты.';

            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                $msg,
                $order->id() ?: ''
            ), $context);

            throw new PaymentGatewayException();
        }
    }

    /**
     * Обработчик notify, вызываемый в качестве вебхука, после попытки оплаты.
     * Может использоваться при возврате платежа
     * @param object $request - объект запроса к вебхуку
     * @return string добавляет статус ответа
     */
    public function onNotify(Request $request)
    {
        parent::onNotify($request);
    }


    /**
     * Получение статуса заказа из VTB API.
     * @param string $orderId - ID заказа
     * @return string Значение статуса заказа, если оно существует, или пустая строка в противном случае.
     */
    private function getPaymentStatus($orderId): string
    {
        // Logging input
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - INPUT: ', [
            'arguments' => [
                'order_id' => $orderId
            ]
        ]);

        $response = $this->getVtbApi()->getOrderInfo($orderId);

        // Logging output
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderInfo - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['status']['value'] ?? '';
    }


    /**
     * Создаем новый экземпляр класса VtbApi с заданными конфигурациями платёжной системы.
     * @return VtbApi The new VtbApi instance.
     */
    protected function getVtbApi(): VtbApi
    {
        return new VtbApi(
            $this->client_id,
            $this->client_secret,
            (bool) $this->mode,
            $this->merchant_authorization ?: ''
        );
    }


    /**
     * Инициализация и настройка объекта класса VtbPayLogger.
     * Эта функция инициализирует и настраивает логгер, используемый плагином VtbPay для ведения журнала.
     * @return void
     */
    private function setVtbpayLogger(): void
    {
        $logger = VtbPayLogger::getInstance();
        $this->logger = $logger
            ->setLogFilePath(dirname(__DIR__, 4) . '/logs/vtbpay-' . date('d-m-Y') . '.log')
            ->setCustomRecording(function($message) use ($logger) {
                if ($this->logging) {
                    $logger->writeToFile($message);
                }
            }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
