<?php

namespace Drupal\commerce_vtbpayment\PluginForm\OffsiteRedirect;

require dirname(__DIR__, 3) . '/vendor/autoload.php';

use Drupal\commerce_payment\PluginForm\PaymentOffsiteForm as BasePaymentOffsiteForm;
use Drupal\Core\Form\FormStateInterface;
use Drupal\commerce_vtbpayment\Classes\Api\VtbApi;
use Drupal\commerce_vtbpayment\Classes\Common\VtbPayLogger;
use Drupal\commerce_vtbpayment\Classes\Exception\VtbPayException;
use Drupal\commerce\Response\NeedsRedirectException;

class VtbPaymentOffsiteForm extends BasePaymentOffsiteForm
{
    private string $client_id;
    private string $client_secret;
    private string $merchant_authorization;
    private bool $mode;
    private bool $logging;

    // Fiscal settings
    private bool $fiscal_enable;
    private string $fiscal_email;
    private string $fiscal_payment_type_delivery;
    private string $fiscal_measure;
    private string $fiscal_tax_type;
    private string $fiscal_payment_type;
    private string $fiscal_subject;

    // Common
    private object $logger;


    /**
     * Устанавливает свойства настроек
     * @param array $configuration - массив настроек
     */
    private function setConfigurationProps(array $configuration)
    {
        // Bool props
        $bool_settings = [
            'logging',
            'mode',
            'fiscal_enable'
        ];
        foreach ($bool_settings as $setting) {
            $bool_setting = $configuration[$setting] ?? false;
            $this->{$setting} = (bool) ($bool_setting === 'test' || $bool_setting === '1');
        }

        // Text props
        $text_settings = [
            'client_id',
            'client_secret',
            'merchant_authorization',
            'fiscal_email',
            'fiscal_payment_type_delivery',
            'fiscal_measure',
            'fiscal_tax_type',
            'fiscal_payment_type',
            'fiscal_subject'
        ];

        foreach ($text_settings as $setting) {
            $this->{$setting} = $configuration[$setting] ?? '';
        }
    }


    /**
     * Формирует запрос и перенаправление на платежную страница.
     * @param array $form - массив формы оплаты
     * @param object $form_state - объект формы оплаты
     * @return array|null Массив, содержащий результат и URL перенаправления, или null, если произошла ошибка.
     */
    public function buildConfigurationForm(array $form, FormStateInterface $form_state) {
        $form = parent::buildConfigurationForm($form, $form_state);
        $payment = $this->entity;
        $order = $payment->getOrder();

        $this->setVtbpayLogger();
        $this->logger->setOption('additionalCommonText', 'payment-' . rand(1111, 9999));

        try {
            $plugin = $payment->getPaymentGateway()->getPlugin();
            $configuration = $plugin->getConfiguration();
            $this->setConfigurationProps($configuration);

            return $this->buildRedirectForm(
                $form,
                $form_state,
                $this->getPayUrl($form),
                [],
                self::REDIRECT_GET
            );
        } catch (\Exception | VtbPayException $e) {
            if ($e instanceof NeedsRedirectException) {
                throw $e;
            }
            // Handle exception and log error
            $context = [
                'file_exception' => $e->getFile(),
                'line_exception' => $e->getLine(),
            ];
            if (method_exists($e, 'getContext')) $context = array_merge($e->getContext(), $context);

            $msg = $e->getMessage() ?: 'Ошибка генерации ссылки оплаты.';

            $this->logger->error(sprintf(
                __FUNCTION__ . ' > VtbPay exception : %s; Order id: %s;',
                $msg,
                $order->id() ?: ''
            ), $context);
            \Drupal::messenger()->addError($msg);
            return null;
        }

  }


    /**
     * Получает URL-адрес платежа для перенаправления.
     * @param array $form - массив формы оплаты для получения адреса возврата
     * @return string URL-адрес платежа.
     */
    private function getPayUrl($form): string
    {
        $payment = $this->entity;
        $order = $payment->getOrder();
        $order_id = $order->id();
        $plugin = $payment->getPaymentGateway()->getPlugin();
        $email = $order->getEmail() ?: $this->fiscal_email;
        $total = floatval($order->getTotalPrice() ? number_format($order->getTotalPrice()->getNumber(),2,'.','') : 0);
        $return_url = $form['#return_url']; // Get return URL

        $items = [];
        if ($this->fiscal_enable) $items = $this->getItems($payment);

        // Logging input
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderLink - INPUT: ', [
            'arguments' => [
                'order_id' => $order_id,
                'email' => $email,
                'time' => time(),
                'total' => $total,
                'return_url' => $return_url,
                'items' => $items
            ]
        ]);

        $response = $this->getVtbApi()->getOrderLink(
            $order_id,
            $email,
            time(),
            $total,
            $return_url,
            false,
            $items
        );

        // Logging output
        $this->logger->debug(
            __FUNCTION__ . ' > getOrderLink - OUTPUT: ', [
            'response' => $response
        ]);

        return $response['object']['payUrl'] ?? '';
    }


    /**
     * Получает массив товаров заказа.
     * @param object $payment - объект с информацией о созданном заказе
     * @return array Массив товаров.
     */
    private function getItems($payment): array
    {
        $order = $payment->getOrder();
        $plugin = $payment->getPaymentGateway()->getPlugin();
        $items = [];
        $key = 1;

        foreach ($order->getItems() as $item) {
            $itemPrice = number_format($item->getUnitPrice()->getNumber(), 2, '.', '');
            $amount = number_format($itemPrice * $item->getQuantity(), 2, '.', '');
            $code = $item->getPurchasedEntity()->getSku() ?: $item->getPurchasedEntityId();

            $items[] = [
                'positionId' => $key,
                'name' => $item->getTitle(),
                'code' => (string) $code,
                'price' => floatval($itemPrice),
                'measure' => (int) $this->fiscal_measure ?: 0,
                'quantity' => (int)$item->getQuantity(),
                'taxParams' => [
                    'taxType' => $this->fiscal_tax_type ?: 'none'
                ],
                'paymentType' => $this->fiscal_payment_type ?: 'full_prepayment',
                'paymentSubject' => (int) $this->fiscal_subject ?: 1,
                'amount' => floatval($amount)
            ];

            $key++;
        }

        foreach ($order->getAdjustments() as $item) {
            if ($item->getType() === 'shipping') {
                $shippingPrice = floatval($item->getAmount()->getNumber());
                if ($shippingPrice <= 0) continue;

                $items[] = [
                    'positionId' => $key,
                    'name' => $item->get_name(),
                    'code' => (string) $item->getPurchasedEntityId()?:'',
                    'price' => $shippingPrice,
                    'measure' => 0,
                    'quantity' => 1,
                    'taxParams' => [
                        'taxType' => $this->fiscal_tax_type ?: 'none'
                    ],
                    'paymentType' => $this->fiscal_payment_type_delivery ?: 'full_prepayment',
                    'paymentSubject' => 4,
                    'amount' => $shippingPrice
                ];

                $key++;
            }
        }

        return $items;
    }


    /**
     * Создаем новый экземпляр класса VtbApi с заданными конфигурациями платёжной системы.
     * @return VtbApi The new VtbApi instance.
     */
    protected function getVtbApi(): VtbApi
    {
        return new VtbApi(
            $this->client_id,
            $this->client_secret,
            (bool) $this->mode,
            $this->merchant_authorization ?: ''
        );
    }


    /**
     * Инициализация и настройка объекта класса VtbPayLogger.
     * Эта функция инициализирует и настраивает логгер, используемый плагином VtbPay для ведения журнала.
     * @return void
     */
    private function setVtbpayLogger(): void
    {
        $logger = VtbPayLogger::getInstance();
        $this->logger = $logger
            ->setLogFilePath(dirname(__DIR__, 3) . '/logs/vtbpay-' . date('d-m-Y') . '.log')
            ->setCustomRecording(function($message) use ($logger) {
                if ($this->logging) {
                    $logger->writeToFile($message);
                }
            }, VtbPayLogger::LOG_LEVEL_DEBUG);
    }
}
